#ifndef Path_h
#define Path_h

#include "Point.h"

class Path
{
	public:
		Path()
		{
			m_root = new Point(-1, -1);
			
			m_next = m_prev = this;
		}

		Path(const Path& copy)
		{
			m_root = new Point(-1, -1);		
			m_next = m_prev = this;

			Point* pt = copy.m_root->GetNext();
			Point* wpt = m_root->GetNext();
			while (pt != copy.m_root)
			{
				Point* ptCopy = new Point(*pt);
	
				Push(ptCopy);

				pt = pt->GetNext();
			}	
		}

		~Path()
		{	
			Point* pt;
			while ((pt = Pop()) != 0)
				delete pt;
		
			if (m_root)
				delete m_root;
		}

		void Push(Point* pt)
		{
			pt->SetPrev(m_root->GetPrev());
			pt->SetNext(m_root);
			m_root->GetPrev()->SetNext(pt);
			m_root->SetPrev(pt);
		}	

		Point* GetTop()
		{
			// act like a stack
			Point* pt = 0;
			if (m_root->GetPrev() != m_root)
			{
				pt = m_root->GetPrev();
			}
			return pt;
		}

		Point* Pop()
		{
			Point* pt = GetTop();
			if (pt)
			{
				pt->GetNext()->SetPrev(pt->GetPrev());
				pt->GetPrev()->SetNext(pt->GetNext());
			}
			return pt;
		}

		// --------------------------------------------------------
		void Print()
		// --------------------------------------------------------
		{
			Point* pt = m_root->GetNext();
			while (pt != m_root)
			{
				printf("  Pt: X=%d, Y=%d\n", pt->GetX(), pt->GetY());
				pt = pt->GetNext();
			}
		}

		void SetPrev(Path* path) { m_prev = path; }
		void SetNext(Path* path) { m_next = path; }
		Path* GetPrev() { return m_prev; }
		Path* GetNext() { return m_next; }

	protected:
		Path* m_prev;
		Path* m_next;

		Point* m_root;
};

#endif