#include "Robot.h"
#include "stdio.h"
#include "string.h"
#include "Board.h"
#include "RobotList.h"
#include "Tile.h"
#include "TargetList.h"
#include "Target.h"
#include "HomeTarget.h"
#include "DeliveryTarget.h"
#include <stdlib.h>
#include <time.h>

/// icfp1.cse.ogi.edu 22008

static int GetRandom(int max)
{
	int num = rand();
	return num%max;
}

// --------------------------------------------------------
Robot::Robot() : m_id(-1), m_maxCarryWeight(-1), m_moneyLeft(-1), 
	m_x(-1), m_y(-1), m_targetList(0), m_curWeight(0), m_callbacks(0)
// --------------------------------------------------------
{
	m_next = m_prev = this;
	m_oneshot[0] = 0;
	m_move[0] = 0;
	srand( (unsigned)time( NULL ) );
}


// --------------------------------------------------------
Robot::Robot(int id, int maxCarryWeight, int initialMoney) : 
	m_id(id), m_maxCarryWeight(maxCarryWeight), m_moneyLeft(initialMoney),
	m_x(-1), m_y(-1), m_targetList(0), m_curWeight(0), m_callbacks(0)
// --------------------------------------------------------
{
	m_next = m_prev = this;
	m_oneshot[0] = 0;
	m_move[0] = 0;
	srand( (unsigned)time( NULL ) );

}

// --------------------------------------------------------
Robot::~Robot()
// --------------------------------------------------------
{
	if (m_targetList)
		delete m_targetList;
}

// --------------------------------------------------------
void Robot::Print()
// --------------------------------------------------------
{
	printf(
"Robot Id: %d\n"
"\tX=%d, Y=%d\n"
"\tMaxCarryWeight=%d, MoneyLeft=%d\n",
		m_id, m_x, m_y, m_maxCarryWeight, m_moneyLeft);
}

// --------------------------------------------------------
void Robot::OnNothing()
// --------------------------------------------------------
{
	m_oneshot[0] = 0;

	// Our robot did nothing
}

// --------------------------------------------------------
void Robot::OnMove(Board* board, int x, int y)
// --------------------------------------------------------
{
	m_oneshot[0] = 0;

	m_x = x;
	m_y = y;
}

// --------------------------------------------------------
void Robot::OnMove(Board* board, char direction)
// --------------------------------------------------------
{
	m_oneshot[0] = 0;

	switch (direction)
	{
		case 'N':
			++m_y;
			break;
		case 'S':
			--m_y;
			break;
		case 'E':
			++m_x;
			break;
		case 'W':
			--m_x;
			break;
	}

	++m_callbacks;
	if (m_callbacks == 2)
	{
		// detect a push so we can recalculate path
		Target* target = GetTarget();
		if (target)
		{
				target->BuildPath(board, GetX(), GetY());
		}
	}
}

// --------------------------------------------------------
void Robot::OnDrop(int idPackage)
// --------------------------------------------------------
{
	// Validate drop
	m_oneshot[0] = 0;

	Package* pkg = m_pkgList.FindByID(idPackage);
	if (pkg)
	{
#ifdef WIN32
		printf("Dropped pkg %d, X=%d, Y=%d, W=%d\n",
			pkg->GetID(), pkg->GetX(), pkg->GetY(), pkg->GetWeight());
		getchar();
#endif

		m_curWeight -= pkg->GetWeight();
		m_pkgList.Unlink(pkg);
	}
}

// --------------------------------------------------------
void Robot::OnPick(int idPackage)
// --------------------------------------------------------
{
	// Validate pick
	m_oneshot[0] = 0;

	Package* pkg = m_pkgListAll.FindByID(idPackage);
	if (pkg)
	{
		Target* target = new DeliveryTarget(*pkg);
		m_targetList->AddHead(target);

		m_pkgList.AddPackage(new Package(*pkg));

		printf("Pickedup %d, X=%d, Y=%d\n", idPackage, pkg->GetX(), pkg->GetY());
	}
}

void Robot::OnTarget(Target* target)
{
	if (target->IsPackage())
	{
		DeliveryTarget* dt = (DeliveryTarget*)target;
		sprintf(m_oneshot, "1 Drop %d\n", dt->m_pkg->GetID());
	}
}

// --------------------------------------------------------
bool Robot::GetRequestLine(char* line)
// --------------------------------------------------------
{
	if (m_oneshot[0] == 0)
	{
		strcpy(line, m_move);
	}
	else
	{
		strcpy(line, m_oneshot);
	}

	return true;
}

// --------------------------------------------------------
bool Robot::PlanMove(Board* board, RobotList* robotList,
	PackageList* pkgList)
// --------------------------------------------------------
{
	if (m_targetList == 0)
	{
		// Build the initial target list of home base's
		BuildTargetList(board);
		m_targetList->Print();
	}

	if (m_oneshot[0] == 0)
	{
		// Determine which target to pursue
		while (1)
		{		
			Target* target = GetTarget();
			if (target)
			{
				// Calculate move
				Path* path = target->GetPath();
				if (path == 0)
				{
					target->BuildPath(board, GetX(), GetY());

					// See if there is a path to target
					path = target->GetPath();	
					if (path == 0)
					{
						// No path, remove target
						m_targetList->Unlink(target);
						continue;
					}
				}

				// Have a path, get next point
				Point* pt = path->Pop();
				if (pt)
				{
					MoveToPoint(*pt);
					delete pt;
				}
				else
				{
					if (GetX() == target->GetX() && m_y == target->GetY())
					{
						OnTarget(target);	
					}
					m_targetList->Unlink(target);
				}

				break;
			}
			else
			{
				// Pick random location
				for (int attempt = 0; attempt < 500; attempt++)	
				{
					int x = GetRandom(board->GetWidth());
					int y = GetRandom(board->GetHeight());
					Tile* tile = board->GetAt(y, x);
					if (tile)
					{
						if (tile->IsHomeBase() || tile->IsPlain())
						{
							Target* target = new HomeTarget(x+1, y+1);
							m_targetList->AddTarget(target);	
							break;
						}
					}
				}
			}
		}
	}

	return true;
}

// --------------------------------------------------------
void Robot::MoveToPoint(Point& pt)
// --------------------------------------------------------
{	
	int dx = (pt.GetX()+1) - m_x;
	int dy = (pt.GetY()+1) - m_y;

	if (dx > 0)
		strcpy(m_move, "1 Move E\n");
	else if (dx < 0)
		strcpy(m_move, "1 Move W\n");
	else if (dy > 0)
		strcpy(m_move, "1 Move N\n");
	else
		strcpy(m_move, "1 Move S\n");

}

// --------------------------------------------------------
void Robot::BuildTargetList(Board* board)
// --------------------------------------------------------
{
	if (m_targetList != 0)
		delete m_targetList;

	m_targetList = new TargetList();
	if (m_targetList != 0)
	{
		for (int row = 0; row < board->GetHeight(); row++)
		{
			for (int col = 0; col < board->GetWidth(); col++)
			{
				Tile* tile = board->GetAt(row, col);
				if (tile)
				{
					if (tile->IsHomeBase())
					{
						Target* target = new HomeTarget(col+1, row+1);
						m_targetList->AddTarget(target);	
					}
				}
			}
		}
	}
}

// --------------------------------------------------------
Target* Robot::GetTarget()
// --------------------------------------------------------
{
	// Get from target list
	Target* target = 0;

	if (m_targetList != 0)
	{
		Target* cur = m_targetList->GetFirst();
		if (cur != m_targetList->GetEnd())
		{
			target = cur;
		}
	}

	return target;
}

void Robot::OnFoundPackages(PackageList* pkgList)
{
	bool bDoPickup = false;
	char cmdLine[MAX_LINE];

	strcpy(cmdLine, "1 Pick");				// TODO: Add in Bid

	int curWeight = m_curWeight;
	Package* pkg = pkgList->GetFirst();
	while (pkg != pkgList->GetEnd())
	{
		if ( (curWeight + pkg->GetWeight()) <= m_maxCarryWeight)
		{
			if (m_pkgListAll.FindByID(pkg->GetID()) == 0)
			{
				curWeight += pkg->GetWeight();

				m_pkgListAll.AddPackage(new Package(*pkg));

				char id[32];
				sprintf(id, " %d", pkg->GetID());
				strcat(cmdLine, id);
				bDoPickup = true;
			}
		}

		pkg = pkg->GetNext();
	}

	if (bDoPickup)
	{
		strcat(cmdLine,"\n");
		strcpy(m_oneshot, cmdLine);
	}
}
