#ifndef Robot_h
#define Robot_h_

#include "PackageList.h"

class Board;
class RobotList;
class PackageList;
class TargetList;
class Target;

#include "Point.h"
#include "Constants.h"

///////////////////////////////////////////////////////////////////////
//                         Robot class
///////////////////////////////////////////////////////////////////////
class Robot
{
	public:
		Robot();

		Robot(int id, int maxCarryWeight, int initialMoney);

		~Robot();

		void Print();

		// Called to allow robot to plan it's next move
		virtual bool PlanMove(Board* board, 
			RobotList* robotList, PackageList* pkgList);

		// Called to retrieve the command to execute
		virtual bool GetRequestLine(char* line);

		/////////////////////////////////////////////
		// Event handlers
		/////////////////////////////////////////////

		// Called when packages found
		virtual void OnFoundPackages(PackageList* pkgList);

		// Called if robot has done nothing for the turn
		virtual void OnNothing();

		// Robot location set to absolute position
		virtual void OnMove(Board* board, int x, int y);

		// Robot moved to relative position
		virtual void OnMove(Board* board, char direction);

		// Robot dropped package
		virtual void OnDrop(int idPackage);

		// Robot picked up package
		virtual void OnPick(int idPackage);

		int GetID() { return m_id; }

		void SetX(int x) { m_x = x; }
		void SetY(int y) { m_y = y; }
		void SetMaxCarryWeight(int maxCarryWeight) { m_maxCarryWeight = maxCarryWeight; }
		void SetMoneyLeft(int initialMoney) { m_moneyLeft = initialMoney; }

		Robot* GetNext() { return m_next; }
		Robot* GetPrev() { return m_prev; }
		void SetNext(Robot* robot) { m_next = robot; }
		void SetPrev(Robot* robot) { m_prev = robot; }
		int GetX() { return m_x; }
		int GetY() { return m_y; }

		// Scan board looking for targets
		void BuildTargetList(Board* board);

	int m_callbacks;

	protected:
		void MoveToPoint(Point& pt);

		Target* GetTarget();

		void OnTarget(Target* target);

	protected:
		int m_id;
		int m_maxCarryWeight;
		int m_moneyLeft;
		int m_curWeight;

		int m_x;
		int m_y;


		TargetList* m_targetList;

		// Packages the robot is currently carrying
		PackageList m_pkgList;

		PackageList m_pkgListAll;

		// List pointers
		Robot* m_next;
		Robot* m_prev;

		char m_move[MAX_LINE];
		char m_oneshot[MAX_LINE];
};

#endif