#include "Server.h"
#include "ctype.h"
#include "stdlib.h"
#include "string.h"

Server::Server(char* addr, int port) : 
	m_Socket(INVALID_SOCKET),
	m_writePos(0), m_readPos(0), m_port(port)
{
	strcpy(m_addr, addr);
#ifdef WIN32
	WSADATA data;
	WSAStartup(2, &data); 
#endif
}

Server::~Server()
{
	if (m_Socket != INVALID_SOCKET)
	{
		shutdown(m_Socket, SD_BOTH);
		closesocket(m_Socket);
	}
}

bool Server::Connect()
{
	bool result = false;

	m_Socket = socket(AF_INET, SOCK_STREAM, 0);
	if (m_Socket != INVALID_SOCKET)
	{
		SOCKADDR_IN sa;
    sa.sin_family = AF_INET;
    sa.sin_port = htons(m_port);

		if (!isdigit(*m_addr)) 
		{
			HOSTENT* hostEnt;
			if ((hostEnt = gethostbyname(m_addr))==0)
			{
				return false;
			}
			memcpy(&sa.sin_addr, hostEnt->h_addr, hostEnt->h_length);
		}
		else 
		{
			sa.sin_addr.s_addr = inet_addr(m_addr);
		}
    
		if (connect(m_Socket, (SOCKADDR*)&sa, sizeof(SOCKADDR_IN)) == 0)
		{
			result = true;
		}
	}

	return result;
}

bool Server::SendCmd(const char* line)
{
	bool result = true;

	int length = strlen(line);
  int pos = 0;
	while (pos < length)
	{
		int count = send(m_Socket, &line[pos], length-pos, 0);
		if (count == -1)
		{
			result = false;
			break;
		}
		pos += count;
	}       

	return result;
}

bool Server::ReadLine(char* line)
{
	int numInBytes;
	int pos = m_readPos;

	bool gotLine = false;

	while (1)
	{
		while (1)
		{
			if (m_buffer[pos] == '\n')
			{
				gotLine = true;
				break;
			}
			else if (pos == m_writePos)
				break;

			++pos;
		}
		
		if (gotLine == true)
			break;

		numInBytes = recv(m_Socket, &m_buffer[m_writePos], MAX_BUFFER-m_writePos, 0);
		if (numInBytes < 0)
			break;

		m_writePos += numInBytes;
	}

	if (gotLine)
	{
		int length = pos-m_readPos;
		memcpy(line, &m_buffer[m_readPos], length);
		line[length] = 0;
		m_readPos = pos+1;
		return true;
	}
	
	return false;
}
