#ifndef Target_h
#define Target_h

#include "PathQueue.h"
#include "Tile.h"

class Target
{
	public:
		Target(int x, int y, bool isPkg) : m_x(x), m_y(y),
			m_path(0), m_isPkg(isPkg)
		{
			m_next = m_prev = this;
		}

		virtual ~Target() 
		{ 
			if (m_path)
				delete m_path;
		}

		bool IsPackage() { return m_isPkg; }

		Target* GetNext() { return m_next; }
		Target* GetPrev() { return m_prev; }
	
		int GetX() { return m_x; }
		int GetY() { return m_y; }

		void SetNext(Target* next) { m_next = next; }
		void SetPrev(Target* prev) { m_prev = prev; }

		// --------------------------------------------------------
		void Print()
		// --------------------------------------------------------
		{
			printf("  Target: X=%d, Y=%d\n", m_x, m_y);
			if (m_path)
			{
				m_path->Print();
			}
		}

		// --------------------------------------------------------
		void ResetBoard(Board* board)
		// --------------------------------------------------------
		{
			for (int y = 0; y < board->GetHeight(); y++)
			{
				for (int x = 0; x < board->GetWidth(); x++)
				{
					Tile* tile = board->GetAt(y, x);
					tile->SetVisited(false);
				}
			}
		}

		// --------------------------------------------------------		
		Path* GetPath()
		// --------------------------------------------------------
		{
			return m_path;
		}

		// --------------------------------------------------------
		bool BuildPath(Board* board, int fromX, int fromY)
		// --------------------------------------------------------
		{
			--fromX;
			--fromY;	// zero base
			
			int toX = m_x-1;
			int toY = m_y-1;

			if (m_path != 0)
			{
				delete m_path;
				m_path = 0;
			}

			PathQueue* master = new PathQueue();			
			if (master)
			{
				ResetBoard(board);

				Path* single = new Path();
				Tile* tile = board->GetAt(fromY, fromX);
				tile->SetVisited(true);
				
				single->Push(new Point(fromX, fromY));
				master->Push(single);
				
				while ((single = master->Pop()) != 0)			// not empty
				{
					Point* curPt = single->GetTop();
					
					for (int dy = curPt->GetY()-1; dy <= curPt->GetY()+1; dy++)
					{
						for (int dx = curPt->GetX()-1; dx <= curPt->GetX()+1; dx++)
						{
							Tile* tile = board->GetAt(dy, dx);
							if (tile)
							{
								if (!IsAdjacent(board, curPt->GetX(), curPt->GetY(), dx, dy))
									continue;

								if (IsOpen(board->GetAt(dy, dx)))
								{
									Point* newPt = new Point(dx, dy);

									if (dx == toX && dy == toY)
									{
										// Found target
										m_path = new Path();
										if (m_path == 0)
											return false;

										m_path->Push(newPt);	// add target to stack

										Point* pt;
										while ((pt = single->Pop()) != 0)
											m_path->Push(pt);

										delete m_path->Pop();	//delete start
										return true;
									}

									tile->SetVisited(true);

									single->Push(newPt);

									master->Push(single);

									delete single->Pop();
								}
							} // if tile
						}
					}
				}	
				delete master;		
			}

			return false;
		}

		// --------------------------------------------------------
		bool IsAdjacent(Board* board, int cx, int cy, int dx, int dy)
		// --------------------------------------------------------
		{
			if ((dx < 0) || (dx >= board->GetWidth()))
				return false;
	
			if ((dy < 0) || (dy >= board->GetHeight()))
				return false;

			if (cx == dx)
			{
				if ((cy-1) == dy)
					return true;

				if ((cy+1) == dy)
					return true;
			}
			else if (cy == dy)
			{
				if ((cx-1) == dx)
					return true;

				if ((cx+1) == dx)
					return true;
			}

			return false;
		}

		// --------------------------------------------------------
		bool IsOpen(Tile* tile)
		// --------------------------------------------------------
		{
			if (tile == 0)
				return false;

			if (tile->GetVisited())
				return false;

			if (tile->IsWall() || tile->IsWater())
				return false;

			return true;
		}

	protected:
		int m_x;
		int m_y;
		Target* m_next;
		Target* m_prev;
		bool m_isPkg;

		Path* m_path;
};

#endif